const AWS = require('aws-sdk');
const fs = require('fs');
//const path = require('path');

const utils = require('./utils');
const CONSTANTS = require('../CONSTANTS')
const { HTTPS, DEBUG } = require('../CONSTANTS')

function init_s3() {
    //{ "accessKeyId": <YOUR_ACCESS_KEY_ID>, "secretAccessKey": <YOUR_SECRET_ACCESS_KEY>, "region": "us-east-1" }
    // AWS.config.loadFromPath('./group_config.json');
    //configuring the AWS environment
    //console.log( "---access-key-->", utils.get_config_value(CONSTANTS.ACCESS_KEY) ) ;
    //console.log( "---secret---key===>", utils.get_config_value(CONSTANTS.SECRET_ACCESS_KEY) );

    let endpointUrl = HTTPS + utils.get_config_value(CONSTANTS.S3_SERVER_ENDPOINT);
    AWS.config.update({
        accessKeyId: utils.get_config_value(CONSTANTS.ACCESS_KEY),
        secretAccessKey: utils.get_config_value(CONSTANTS.SECRET_ACCESS_KEY),
        endpoint: endpointUrl
    });
    let s3 = new AWS.S3();
    return s3;
}
// -------------------
function download_file_from_s3(bucket_name, s3_file_obj_name, local_file_path) {
    let s3 = init_s3();

    var fileStream = fs.createWriteStream(local_file_path);
    var s3Stream = s3.getObject({ Bucket: bucket_name, Key: s3_file_obj_name }).createReadStream();

    s3Stream.on('error', function (err) {
        console.error(err); // NoSuchKey: The specified key does not exist
    });

    s3Stream.pipe(fileStream).on('error', function (err) {
        // capture any errors that occur when writing data to the file
        if (DEBUG) console.log(err); else console.log('Error-37: Error in downlading file ');
    }).on('close', function () {
        console.log(' \n\n Download Complete: ', local_file_path, "  \n\n ");
    });
}
// -------------------
function upload_to_s3(local_file_path, bucket_name, s3_file_obj_name, public_permision, callback) {
    let s3 = init_s3();
    var params = {
        Bucket: bucket_name,
        Body: fs.createReadStream(local_file_path),
        Key: s3_file_obj_name
    };

    if (public_permision) {
        params['ACL'] = 'public-read';
    }
    //console.log("53", local_file_path,  bucket_name, s3_file_obj_name, "public_permision:", public_permision)
    s3.upload(params, function (err, data) {
        //handle error
        if (err) {
            if (DEBUG) console.log(err); else console.log('Error-59: Error in uploading file ');
        }

        //success
        if (data) {
            callback(data);
        }
    });
}

function delete_file_from_s3(bucket_name, s3_file_path, callback) {
    let s3 = init_s3();

    var params = {
        Bucket: bucket_name,
        Delete: {
            Objects: [
                {
                    Key: s3_file_path
                }
            ],
        },
    };

    s3.deleteObjects(params, function (err, data) {
        //handle error
        if (err) {
            if (DEBUG) console.log(err); else console.log('Error-87: Error in deleting file ');
        }

        //success
        if (data) {
            callback(data);
        }          
    });
}

module.exports = {
    init_s3,
    upload_to_s3,
    download_file_from_s3,
    delete_file_from_s3
}